#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAIN_C
#include "exitfail.h"

#define SERIAL_BAUDRATE B115200
#define BUF_SIZE 2048

static int serial_fd = -1;
static struct termios old_tio;

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

/**** sample:oki920 command: begin ****/
static unsigned char cmd_toka_co2[13] = {0x7e, 0x11, 0x00, 0x08, 
0x01, 0x03, 0x00, 0x44, 0x00, 0x01, 0xc4, 0x1f, 
0x7e};

static unsigned char cmd_toka_tmp[13] = {0x7e, 0x11, 0x00, 0x08, 
0x01, 0x03, 0x00, 0x45, 0x00, 0x01, 0x95, 0xdf, 
0x7e};

static unsigned char cmd_toka_hum[13] = {0x7e, 0x11, 0x00, 0x08, 
0x01, 0x03, 0x00, 0x46, 0x00, 0x01, 0x65, 0xdf, 
0x7e};
/**** sample: oki920 command: end ****/

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;
	
	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

static void restore_serial(void)
{
	int ret;

	ret = tcsetattr(serial_fd, TCSANOW, &old_tio);
	if(ret < 0) exitfail_errno("tcsetattr");
}

static void setup_serial(int fd)
{
	struct termios tio;
	int ret;

	ret = tcgetattr(fd, &old_tio);
	if(ret) exitfail_errno("tcsgetattr");

	if(atexit(restore_serial)) exitfail_errno("atexit");

	memset(&tio, 0, sizeof(tio));
	tio.c_iflag = IGNBRK | IGNPAR;
	tio.c_cflag = CS8 | CLOCAL | CREAD;
	tio.c_cc[VTIME] = 0;
	tio.c_cc[VMIN] = 1;
	ret = cfsetspeed(&tio, SERIAL_BAUDRATE);
	if(ret < 0) exitfail_errno("cfsetspeed");

	ret = tcflush(fd, TCIFLUSH);
	if(ret < 0) exitfail_errno("tcflush");

	ret = tcsetattr(fd, TCSANOW, &tio);
	if(ret) exitfail_errno("tcsetattr");
}

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};
//	char buf[BUF_SIZE]; // original:co
	ssize_t ret, len, wrlen;
/**** sample:variable:begin ****/
       unsigned char buf[BUF_SIZE];
	unsigned char buf1[BUF_SIZE];
       int cnt_char;
       int cnt_head;
	int cnt_esp;
	int data_len;
	int i;
	int data_co2;
	float data_tmp;
	float data_hum;
	ssize_t analen;
	FILE *fp;

	unsigned char proc_id;
	int cnt_cmd;
/**** sample:variable:end ****/

	exitfail_init();

	if(argc < 2) {
		printf("Usage: %s <device>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	serial_fd = open(argv[1], O_RDWR | O_NOCTTY);
	if(serial_fd < 0) exitfail_errno("open");

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	setup_serial(serial_fd);

/**** sample:initial value:begin ****/
	cnt_char = 0;
	cnt_head = 0;
	cnt_esp = 0;
	data_len = 0;
	data_co2 = 0;
	data_tmp = 0;
	data_hum = 0;

	proc_id = 0x00;
	cnt_cmd = 0;
/**** sample:initial value:end ****/

	ret = write(serial_fd, cmd_toka_co2, 13);
	printf("\n***** read co2 ****\n");
	if (ret < 0) exitfail_errno("write");

/*
	ret = write(serial_fd, cmd_toka_tmp, 13);
	printf("\n***** read temperature ****\n");
	if (ret < 0) exitfail_errno("write");

	ret = write(serial_fd, cmd_toka_hum, 13);
	printf("\n***** read humidity ****\n");
	if (ret < 0) exitfail_errno("write");
*/
	while(!terminated) {
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) {
			if(errno == EINTR) continue;
			exitfail_errno("read");
		}
		len = ret;

/**** sample:receive:begin ****/
		for(analen = 0; analen < len; analen++) {
			if(cnt_head == 0 && buf[analen] == 0x7e) {
				buf1[cnt_esp] = buf[analen];
				printf("0x%02x ",  buf1[cnt_esp]);
				cnt_esp++;
				cnt_head++;
			}
			else if(cnt_head == 1) {
				if(buf[analen] == 0x7e) {
					/**** counter reset for sync shift ****/
					cnt_head = 0;
					cnt_esp = 0;
					data_len = 0;

					buf1[cnt_esp] = buf[analen];
					printf("0x%02x ",  buf1[cnt_esp]);
					cnt_esp++;
					cnt_head++;
				}
				else {
					buf1[cnt_esp] = buf[analen];
					proc_id = buf[analen];
					printf("0x%02x ",  buf1[cnt_esp]);
					cnt_esp++;
					cnt_head++;
				}
			}
			else if(cnt_head == 2) {
				buf1[cnt_esp] = buf[analen];
				printf("0x%02x ",  buf1[cnt_esp]);
				cnt_esp++;
				cnt_head++;
			}
			else if(cnt_head == 3) {
				buf1[cnt_esp] = buf[analen];
				printf("0x%02x ",  buf1[cnt_esp]);
				cnt_esp++;
				cnt_head++;
				data_len = buf1[2] * 256 + buf1[3];
			}
			else if(cnt_head > 3 && cnt_esp <= (3 + data_len + 1)) {
				buf1[cnt_esp] = buf[analen];
				printf("0x%02x ",  buf1[cnt_esp]);
				
				if(cnt_esp == (3 + data_len + 1)) {
					fp = fopen("/root/log_oki920co2.txt", "w");
/*
					for(i = 0; i <= (3 + data_len + 1); i++) 
						fprintf(fp, "buf[%d]: %02x ", i, buf1[i]); 
*/

					data_co2 = buf1[7] * 256 + buf1[8];
					fprintf(fp, "%d[ppm]\n", data_co2); 

					fclose(fp);

					printf("\nreceive finish\n");

					cnt_head = 0;
					cnt_esp = 0;
					data_len = 0;

					exit(0);
					/* シリアルポートに書き込み */
//					if(cnt_cmd == 0) {
//						ret = write(serial_fd, cmd_toka, 13);
//						printf("\n***** start toka ****\n");
//						if (ret < 0) exitfail_errno("write");
//						cnt_cmd++;
//					}
				}
				else cnt_esp++;
			}
			else {
				cnt_head = 0;
				cnt_esp = 0;
				data_len = 0;
			}
		}
	}
	return EXIT_SUCCESS;
}

